precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;

float mosaicSize = 1.0/25.0;
vec2 center = vec2(0.5);
void main() {

    float TR = 0.866025;
    float TB = 1.5;
    vec2 cha = textureCoordinate - center;
    if(ratio>1.0){
        cha.x*=ratio;
    }else{
        cha.y/=ratio;
    }

    float PI6 = 0.523599;
    //第一步找到矩形顶点
    float indexX = floor(cha.x / TB / mosaicSize);
    float indexY = floor(cha.y / TR / mosaicSize);
    vec2 v1, v2, result;
    bool isx = fract(indexX / 2.0) == 0.0;
    bool isy = fract(indexY / 2.0) == 0.0;
    //第二步找到矩形中可用于取色的顶点
    if (isx) {
        if (isy) {
            v1 = vec2(indexX * mosaicSize * TB, indexY * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
        } else {
            v1 = vec2(indexX * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, indexY * mosaicSize * TR);
        }
    } else {
        if (isy) {
            v1 = vec2(indexX * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, indexY * mosaicSize * TR);
        } else {
            v1 = vec2(indexX * mosaicSize * TB, indexY * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
        }
    }
    //第三步通过比较远近来确定纹素的坐标
    float s1 = sqrt(pow(cha.x - v1.x, 2.0) + pow(cha.y - v1.y, 2.0));
    float s2 = sqrt(pow(cha.x - v2.x, 2.0) + pow(cha.y - v2.y, 2.0));
    if (s1 < s2) {
        result = v1;
    } else {
        result = v2;
    }
    //2 计算出夹角
    float a =  atan(cha.x - result.x , cha.y - result.y);


    //3 计算出每一个三角形中心点的纹理坐标
    vec2 area1 = vec2(result.x, result.y - mosaicSize * TR / 2.0);
    vec2 area2 = vec2(result.x + mosaicSize / 2.0, result.y - mosaicSize * TR / 2.0);
    vec2 area3 = vec2(result.x + mosaicSize / 2.0, result.y + mosaicSize * TR / 2.0);
    vec2 area4 = vec2(result.x, result.y + mosaicSize * TR / 2.0);
    vec2 area5 = vec2(result.x - mosaicSize / 2.0, result.y + mosaicSize * TR / 2.0);
    vec2 area6 = vec2(result.x - mosaicSize / 2.0, result.y - mosaicSize * TR / 2.0);
    //4 通过判断得到当前点隶属的三角形中心点
    vec2 vn;
    if (a >= -PI6 && a < PI6) {
        vn = area1;
    } else if (a >= PI6 && a < 3.0 * PI6) {
        vn = area2;
    } else if (a >= 3.0 * PI6 && a < 5.0 * PI6) {
        vn = area3;
    } else if ((a >= PI6 * 5.0 && a <= PI6 * 6.0) || (a < -PI6 * 5.0 && a > -PI6 * 6.0)) {
        vn = area4;
    } else if (a < -PI6 * 3.0 && a >= -PI6 * 5.0) {
        vn = area5;
    } else if (a <= -PI6 && a > -PI6 * 3.0) {
        vn = area6;
    }
    if(ratio>1.0){
        vn.x/=ratio;
    }else{
        vn.y*=ratio;
    }
    vn+=center;

    gl_FragColor = texture2D(inputImageTexture, vn);
}

