precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;

float mosaicSize = 1.0/40.0;
vec2 center = vec2(0.5);
void main() {

    float TR = 0.866025;
    float TB = 1.5;
    vec2 cha = textureCoordinate - center;
    if(ratio>1.0){
        cha.x*=ratio;
    }else{
        cha.y/=ratio;
    }

    //第一步找到矩形顶点
    float indexX = floor(cha.x / TB / mosaicSize);
    float indexY = floor(cha.y / TR / mosaicSize);
    vec2 v1, v2, result;
    bool isx = fract(indexX / 2.0) == 0.0;
    bool isy = fract(indexY / 2.0) == 0.0;
    //第二步找到矩形中可用于取色的顶点
    if (isx) {
        if (isy) {
            v1 = vec2(indexX * mosaicSize * TB, indexY * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
        } else {
            v1 = vec2(indexX * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, indexY * mosaicSize * TR);
        }
    } else {
        if (isy) {
            v1 = vec2(indexX * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, indexY * mosaicSize * TR);
        } else {
            v1 = vec2(indexX * mosaicSize * TB, indexY * mosaicSize * TR);
            v2 = vec2((indexX + 1.0) * mosaicSize * TB, (indexY + 1.0) * mosaicSize * TR);
        }
    }

    //第三步通过比较远近来确定纹素的坐标
    float s1 = sqrt(pow(cha.x - v1.x, 2.0) + pow(cha.y - v1.y, 2.0));
    float s2 = sqrt(pow(cha.x - v2.x, 2.0) + pow(cha.y - v2.y, 2.0));
    if (s1 < s2) {
        result = v1;
    } else {
        result = v2;
    }
    if(ratio>1.0){
        result.x/=ratio;
    }else{
        result.y*=ratio;
    }
    result+=center;
    gl_FragColor = texture2D(inputImageTexture, result);
}
