//https://www.shadertoy.com/view/MsGGzW
precision highp float;
varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float progress;
uniform float ratio;

const float pi = 3.1415926535897;
const float studRad = 0.3;
const float studHeight = 0.2;
const float studBorder = 0.03;
const vec2 halfXY = vec2(0.5, 0.5);
float num = 40.0;
vec2 tocenter;

// Photoshop/GIMP style multiplicative-ish style filter.
float blend(float a, float b) {
    return (a < 0.5) ? 2. * a * b : (1. - 2. * (1. - a) * (1. - b));
}

vec4 blend(vec4 a, vec4 b) {
    return vec4(blend(a.x, b.x),
                blend(a.y, b.y),
                blend(a.z, b.z),
                1);
}



// Essentially convolves a virtual pixel with the stud pattern.
vec4 brickify(vec4 baseColor) {
    vec2 subGrid = tocenter;
    float rad = length(subGrid);

    float lightFactor = smoothstep(-studRad, studRad, subGrid.y);

    float pixelsPerGrid = 30.0;

    float alpha= (abs(rad - (studRad - 0.5 * studBorder)) <= 0.5 * studBorder) ?
    0.5 * clamp(pixelsPerGrid * (0.5 * studBorder - abs(rad - (studRad - 0.5 * studBorder))), 0., 1.) : 0.;
    vec4 borderColor = vec4(lightFactor, lightFactor, lightFactor,alpha);


    float rightFactor = 0.3;
    vec4 rightColor = vec4(rightFactor, rightFactor, rightFactor,
                           (0.5 - subGrid.x) <= studBorder ? 0.3 : 0.);

    float bottomFactor = 0.3;
    vec4 bottomColor = vec4(bottomFactor, bottomFactor, bottomFactor,
                            (0.5 + subGrid.y) <= studBorder ? 0.3 : 0.);

    vec4 fragColor = vec4(0.5, 0.5, 0.5, 1);
    fragColor = mix(fragColor, borderColor, borderColor.w);

    if (abs(subGrid.x) <= studRad - 1. / pixelsPerGrid && subGrid.y <= 0.) {
        float angle = acos(subGrid.x / studRad);
        float yInt = -sin(angle) * studRad;

        float vFac = 0.5 * smoothstep(0., studHeight, (yInt - subGrid.y) * 1.5 * exp(-pow(subGrid.x, 2.)));
        float sFac = vFac;
        vec4 shadowColor = vec4(sFac, sFac, sFac, subGrid.y <= yInt ? 1.: clamp(1. - pixelsPerGrid * abs(rad - studRad),  0., 1.));
        fragColor = mix(fragColor, shadowColor, 0.5 * shadowColor.w);
    }


    fragColor = mix(fragColor, rightColor, rightColor.w);
    fragColor = mix(fragColor, bottomColor, bottomColor.w);

    fragColor = blend(baseColor, fragColor);

    return fragColor;
}


void main() {
    vec2 usenum = vec2(num * ratio, num);
    vec2 pos = textureCoordinate;
    pos *= usenum;
    tocenter.x = 0.5-fract(pos.x);
    tocenter.y = 0.5-fract(pos.y);

    pos.x = floor(pos.x) + 0.5;
    pos.y = floor(pos.y) + 0.5;
    pos /= usenum;
    vec4 color = texture2D(inputImageTexture, pos);
    gl_FragColor = brickify(color);
}